# Copyright (C) 2004 Laurent Sansonetti
#
# Tengen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Tengen is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Tengen; see the file COPYING.  If not,
# write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

require 'thread'

module Tengen
    class GTP
        class Command
            attr_reader :name, :arity, :shortcuts
            def initialize(name, arity=0, *shortcuts)
                @name = name
                @arity = arity
                @shortcuts = (shortcuts.empty?) ? [ name ] : shortcuts
            end
        end
       
        COMMANDS = [
            Command.new("boardsize", 1, "boardsize=", "set_boardsize"),
            Command.new("captures", 1),
            Command.new("clear_board"),
            Command.new("final_score"),
            Command.new("fixed_handicap", 1),
            Command.new("genmove", 1),
            Command.new("known_command", 1, "know_command?"),
            Command.new("komi", 1, "komi=", "set_komi"),
            Command.new("list_commands", 0, "commands"),
            Command.new("list_stones", 1, "stones"),
            Command.new("name"),
            Command.new("place_free_handicap", 1),
            Command.new("play", 2),
            Command.new("protocol_version"),
            Command.new("quit"),
            Command.new("set_free_handicap", -1),
            Command.new("showboard"),
            Command.new("undo"),
            Command.new("version")
        ]
    	
        class Vertex
            attr_reader :letter, :number
            def initialize(vertex)
                if md = /^(\w)(\d+)$/.match(vertex)
                    @letter, @number = md[1], md[2].to_i
                else
                    raise "Invalid vertex ``#{vertex}''" 
                end
            end
        
            def to_s
                "#{letter}#{number}"
            end
    
            def ==(vertex)
                self.to_s == vertex.to_s
            end

            def <=>(vertex)
                [letter, number] <=> [vertex.letter, vertex.number]
            end
        end

        class Error < StandardError; end

        def initialize(io=nil)
            io = IO.popen("gnugo --quiet --mode gtp", "w+") unless io
            @io = io 
            @id = 0
            @gtp_command = Mutex.new
            if block_given?
                yield self
                quit
            end
        end
    
        def method_missing(id, *args)
            method = id.id2name
            unless cmd = COMMANDS.find { |cmd| cmd.shortcuts.include?(method) }
                raise "Command ``#{method}'' not recognized"
            end
            unless cmd.arity == -1 or args.size == cmd.arity
                raise "Wrong number of parameters (#{args.size} " \
                      "for #{cmd.arity})" 
            end
            invoke(cmd.name, *args)
        end
    
        def pass(color)
            invoke("play", color, "pass")
        end
    
        #######
        private 
        #######
    
        def invoke(command, *args)
            @gtp_command.synchronize do
                @id += 1
                gtp_command = "#{@id} #{command} #{args.join(' ')}"
                @io.puts(gtp_command)
                resp = @io.gets
                Tengen.debug(gtp_command, "=> '#{resp.inspect}'")
                if md = /^([?=])(\d+)\s(.*)$/.match(resp)
                    fc, id, msg = md[1..3]
                    unless @id == id.to_i
                        raise Error, "Invalid command id ``#{id}''"
                    end
                    case fc 
                        when "?"
                            until @io.gets == "\n"; end
                            raise Error, msg.capitalize 
          
                        when "="
                            data = [ msg ] 
                            until (msg = @io.gets) == "\n"
                                data << msg 
                            end
                            data.map! { |x| convert(x) } 
                            data.length == 1 ? data.first : data.compact
        
                        else
                            raise Error, "Invalid first character ``#{fc}''" 
                    end
                elsif resp != "\n"
                    raise Error, "Invalid response ``#{resp}''"
                end 
            end
        end
    
        def convert(something)
            case something.strip
                when ""         then nil
                when "true"     then true
                when "false"    then false
                when /^\d+$/    then something.to_i
                
                when /^(\w\d+)$/
                    Vertex.new(something)    

                when /^(\w\d+\s?)+$/
                    something.split(/\s/).map { |x| Vertex.new(x) }
                        
                else 
                    something
            end
        end
    end
end
