# Copyright (C) 2004 Laurent Sansonetti
#
# Tengen is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Tengen is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with Tengen; see the file COPYING.  If not,
# write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

module Tengen::UI
    class ReallyQuitDialog < AlertDialog
        def initialize(parent)
            super(parent, "Really quit?", Gtk::Stock::DIALOG_QUESTION, 
                  [[Gtk::Stock::CANCEL, Gtk::Dialog::RESPONSE_CANCEL],
                   [Gtk::Stock::QUIT, Gtk::Dialog::RESPONSE_OK]], 
                  'The game is not yet finished.  Are you sure you want to ' \
                  'quit now and abort it?')
            self.default_response = Gtk::Dialog::RESPONSE_CANCEL
            show_all and @response = run
            destroy
        end

        def quit?
            @response == Gtk::Dialog::RESPONSE_OK
        end
    end
    
    class MainApp < Gnome::App
        def initialize
            super(Tengen::TITLE, Tengen::VERSION)
            build_ui
            start_game
            signal_connect('destroy') { Gtk.main_quit }
        end

        def update(game, player, action)
            if action.undo?
                @statusbar.status = "Undid last move..."
            else
                status = "#{player.color} "
                if action.pass?
                    status += "passed"
                else
                    status += "played #{action.vertex}"
                end
                status += " - #{game.other(player).color} is thinking..."
                @statusbar.status = status
            end
            sensitize_ui
        end
        
        #######
        private
        #######

        def default_game
            game = Tengen::Game.new
            game.add_observer(self)
            game.black_player = Tengen::Player::Human.new
            game.white_player = Tengen::Player::Computer.new

            # The default boardsize (19) is too large for screen
            # resolutions with a height <= 768.
            game.boardsize = 15 if Gdk::Screen.default.height <= 768
            return game
        end

        def start_game(game=nil)
            @game = (game or default_game)
            @box.remove(@board) if @board
            @board = Board.new(self, @game).show
            @box << @board
            @statusbar.status = "New game started!"
            @game.when_finished do
                @statusbar.status = "#{@game.winner.color} won! " \
                                    "(final score is #{@game.score})"
                @board.freeze
                sensitize_ui
            end
            self.title = "#{@game.first_player} vs " \
                         "#{@game.other(@game.first_player)} - " \
                       + Tengen::TITLE
            @game.start
            sensitize_ui
        end
        
        def build_ui
            on_new_game = \
                proc { NewGameDialog.new(self) { |game| start_game(game) } } 
            on_undo = proc do 
                begin
                    @game.undo
                    @board.redraw_from_engine
                rescue Tengen::GTP::Error => error
                    ErrorDialog.new(@parent, error.message, 
                                    "Moves history is empty.  It is " \
                                    "therefore not possible to undo.")
                end
             end
            on_pass = proc { @game.current_player.pass }
            on_resign = proc { @game.current_player.resign }
            on_quit = proc do 
                if @game.finished? or ReallyQuitDialog.new(self).quit?
                    Gtk.main_quit
                end
            end
            on_about = proc { AboutDialog.new(self) } 

            regular_actions = [
                ["GameMenu", nil, "_Game"],
                ["New", Gtk::Stock::NEW, "_New Game...", "<control>N", nil, 
                 on_new_game],
                ["UndoMove", Gtk::Stock::UNDO, "_Undo Move", "<control>Z", 
                 nil, on_undo],
                ["PassMove", Gtk::Stock::REDO, "_Pass Move", "<control>P", 
                 nil, on_pass],
                ["Resign", Gtk::Stock::STOP, "_Resign", nil, nil, on_resign],
                ["Quit", Gtk::Stock::QUIT, "_Quit", "<control>Q", nil, 
                 on_quit],
                ["ViewMenu", nil, "_View"],
                ["HelpMenu", nil, "_Help"],
                ["About", Gnome::Stock::ABOUT, "_About", nil, nil, on_about]
            ]
          
            on_view_toolbar = proc do |actiongroup, action| 
                @toolbar.parent.visible = action.active?
            end
            on_view_statusbar = proc do |actiongroup, action| 
                @statusbar.visible = action.active?
            end
          
            toggle_actions = [
              ["ShowToolbar", nil, "_Toolbar", nil, nil, on_view_toolbar, 
               true],
              ["ShowStatusbar", nil, "_Statusbar", nil, nil, 
               on_view_statusbar, true]
            ]
         
            @actiongroup = Gtk::ActionGroup.new("actions")
            @actiongroup.add_actions(regular_actions)
            @actiongroup.add_toggle_actions(toggle_actions)
         
            uimanager = Gtk::UIManager.new
            uimanager.insert_action_group(@actiongroup, 0)
            uimanager.add_ui(File.join(Tengen::Config::DATA_DIR,
                                       "ui", "ui.xml"))
            self.add_accel_group(uimanager.accel_group)

            self.resizable = false
            self.contents = @box = Gtk::HBox.new
            self.toolbar = @toolbar = uimanager.get_widget("/MainToolBar")
            self.menus = uimanager.get_widget("/MainMenuBar")
            self.statusbar = @statusbar = 
              Gnome::AppBar.new(false, true, Gnome::PREFERENCES_USER)
        end

        def sensitize_ui
            %w{UndoMove PassMove Resign}.each do |action_name|
                action = @actiongroup.get_action(action_name)
                action.sensitive = !(@game.current_player.computer? or 
                                     @game.finished?)
            end
        end
    end
end
